@@ -35,11 +35,14 @@ from airflow.operators.empty import EmptyOperator
 from airflow.sensors.base import BaseSensorOperator
 from airflow.utils.file import correct_maybe_zipped
 from airflow.utils.helpers import build_airflow_url_with_query
-from airflow.utils.session import provide_session
+from airflow.utils.session import NEW_SESSION, provide_session
+from airflow.utils.sqlalchemy import tuple_in_condition
 from airflow.utils.state import State
 
 if TYPE_CHECKING:
-    from sqlalchemy.orm import Query
+    from sqlalchemy.orm import Query, Session
+
+    from airflow.utils.context import Context
 
 
 class ExternalDagLink(BaseOperatorLink):
@@ -72,6 +75,12 @@ class ExternalTaskSensor(BaseSensorOperator):
     until the sensor times out (thus giving you time to retry the external task
     without also having to clear the sensor).
 
+    By default, the ExternalTaskSensor will not skip if the external task skips.
+    To change this, simply set ``skipped_states=[State.SKIPPED]``. Note that if
+    you are monitoring multiple tasks, and one enters error state and the other
+    enters a skipped state, then the external task will react to whichever one
+    it sees first. If both happen together, then the failed state takes priority.
+
     It is possible to alter the default behavior by setting states which
     cause the sensor to fail, e.g. by setting ``allowed_states=[State.FAILED]``
     and ``failed_states=[State.SUCCESS]`` you will flip the behaviour to get a
@@ -82,17 +91,24 @@ class ExternalTaskSensor(BaseSensorOperator):
     if the external task enters a failed state and ``soft_fail == True`` the
     sensor will _skip_ rather than fail. As a result, setting ``soft_fail=True``
     and ``failed_states=[State.SKIPPED]`` will result in the sensor skipping if
-    the external task skips.
+    the external task skips. However, this is a contrived example - consider
+    using ``skipped_states`` if you would like this behaviour. Using
+    ``skipped_states`` allows the sensor to skip if the target fails, but still
+    enter failed state on timeout. Using ``soft_fail == True`` as above will
+    cause the sensor to skip if the target fails, but also if it times out.
 
     :param external_dag_id: The dag_id that contains the task you want to
-        wait for
+        wait for. (templated)
     :param external_task_id: The task_id that contains the task you want to
-        wait for.
-    :param external_task_ids: The list of task_ids that you want to wait for.
+        wait for. (templated)
+    :param external_task_ids: The list of task_ids that you want to wait for. (templated)
         If ``None`` (default value) the sensor waits for the DAG. Either
         external_task_id or external_task_ids can be passed to
         ExternalTaskSensor, but not both.
+    :param external_task_group_id: The task_group_id that contains the task you want to
+        wait for. (templated)
     :param allowed_states: Iterable of allowed states, default is ``['success']``
+    :param skipped_states: Iterable of states to make this task mark as skipped, default is ``None``
     :param failed_states: Iterable of failed or dis-allowed states, default is ``None``
     :param execution_delta: time difference with the previous execution to
         look at, the default is the same logical date as the current task or DAG.
@@ -110,7 +126,7 @@ class ExternalTaskSensor(BaseSensorOperator):
         or DAG does not exist (default value: False).
     """
 
-    template_fields = ["external_dag_id", "external_task_id", "external_task_ids"]
+    template_fields = ["external_dag_id", "external_task_id", "external_task_ids", "external_task_group_id"]
     ui_color = "#19647e"
     operator_extra_links = [ExternalDagLink()]
 
@@ -122,6 +138,7 @@ class ExternalTaskSensor(BaseSensorOperator):
         external_task_ids: Collection[str] | None = None,
         external_task_group_id: str | None = None,
         allowed_states: Iterable[str] | None = None,
+        skipped_states: Iterable[str] | None = None,
         failed_states: Iterable[str] | None = None,
         execution_delta: datetime.timedelta | None = None,
         execution_date_fn: Callable | None = None,
@@ -130,42 +147,51 @@ class ExternalTaskSensor(BaseSensorOperator):
     ):
         super().__init__(**kwargs)
         self.allowed_states = list(allowed_states) if allowed_states else [State.SUCCESS]
+        self.skipped_states = list(skipped_states) if skipped_states else []
         self.failed_states = list(failed_states) if failed_states else []
 
-        total_states = set(self.allowed_states + self.failed_states)
+        total_states = set(self.allowed_states + self.skipped_states + self.failed_states)
 
-        if set(self.failed_states).intersection(set(self.allowed_states)):
+        if len(total_states) != len(self.allowed_states) + len(self.skipped_states) + len(self.failed_states):
             raise AirflowException(
-                f"Duplicate values provided as allowed "
-                f"`{self.allowed_states}` and failed states `{self.failed_states}`"
+                "Duplicate values provided across allowed_states, skipped_states and failed_states."
             )
 
+        # convert [] to None
+        if not external_task_ids:
+            external_task_ids = None
+
+        # can't set both single task id and a list of task ids
         if external_task_id is not None and external_task_ids is not None:
             raise ValueError(
                 "Only one of `external_task_id` or `external_task_ids` may "
-                "be provided to ExternalTaskSensor; not both."
+                "be provided to ExternalTaskSensor; "
+                "use external_task_id or external_task_ids or external_task_group_id."
             )
 
+        # since both not set, convert the single id to a 1-elt list - from here on, we only consider the list
         if external_task_id is not None:
             external_task_ids = [external_task_id]
 
-        if external_task_group_id and external_task_ids:
+        if external_task_group_id is not None and external_task_ids is not None:
             raise ValueError(
-                "Values for `external_task_group_id` and `external_task_id` or `external_task_ids` "
-                "can't be set at the same time"
+                "Only one of `external_task_group_id` or `external_task_ids` may "
+                "be provided to ExternalTaskSensor; "
+                "use external_task_id or external_task_ids or external_task_group_id."
             )
 
+        # check the requested states are all valid states for the target type, be it dag or task
         if external_task_ids or external_task_group_id:
             if not total_states <= set(State.task_states):
                 raise ValueError(
-                    f"Valid values for `allowed_states` and `failed_states` "
-                    f"when `external_task_id` or `external_task_ids` or `external_task_group_id` "
+                    "Valid values for `allowed_states`, `skipped_states` and `failed_states` "
+                    "when `external_task_id` or `external_task_ids` or `external_task_group_id` "
                     f"is not `None`: {State.task_states}"
                 )
 
         elif not total_states <= set(State.dag_states):
             raise ValueError(
-                f"Valid values for `allowed_states` and `failed_states` "
+                "Valid values for `allowed_states`, `skipped_states` and `failed_states` "
                 f"when `external_task_id` and `external_task_group_id` is `None`: {State.dag_states}"
             )
 
@@ -194,7 +220,8 @@ class ExternalTaskSensor(BaseSensorOperator):
         return dttm if isinstance(dttm, list) else [dttm]
 
     @provide_session
-    def poke(self, context, session=None):
+    def poke(self, context: Context, session: Session = NEW_SESSION) -> bool:
+        # delay check to poke rather than __init__ in case it was supplied as XComArgs
         if self.external_task_ids and len(self.external_task_ids) > len(set(self.external_task_ids)):
             raise ValueError("Duplicate task_ids passed in external_task_ids parameter")
 
@@ -217,12 +244,17 @@ class ExternalTaskSensor(BaseSensorOperator):
                 serialized_dttm_filter,
             )
 
+        if self.external_dag_id and not self.external_task_group_id and not self.external_task_ids:
+            self.log.info(
+                "Poking for DAG '%s' on %s ... ",
+                self.external_dag_id,
+                serialized_dttm_filter,
+            )
+
         # In poke mode this will check dag existence only once
         if self.check_existence and not self._has_checked_existence:
             self._check_for_existence(session=session)
 
-        count_allowed = self.get_count(dttm_filter, session, self.allowed_states)
-
         count_failed = -1
         if self.failed_states:
             count_failed = self.get_count(dttm_filter, session, self.failed_states)
@@ -257,6 +289,31 @@ class ExternalTaskSensor(BaseSensorOperator):
                     )
                 raise AirflowException(f"The external DAG {self.external_dag_id} failed.")
 
+        count_skipped = -1
+        if self.skipped_states:
+            count_skipped = self.get_count(dttm_filter, session, self.skipped_states)
+
+        # Skip if anything in the list has skipped. Note if we are checking multiple tasks and one skips
+        # before another errors, we'll skip first.
+        if count_skipped > 0:
+            if self.external_task_ids:
+                raise AirflowSkipException(
+                    f"Some of the external tasks {self.external_task_ids} "
+                    f"in DAG {self.external_dag_id} reached a state in our states-to-skip-on list. Skipping."
+                )
+            elif self.external_task_group_id:
+                raise AirflowSkipException(
+                    f"The external task_group '{self.external_task_group_id}' "
+                    f"in DAG {self.external_dag_id} reached a state in our states-to-skip-on list. Skipping."
+                )
+            else:
+                raise AirflowSkipException(
+                    f"The external DAG {self.external_dag_id} reached a state in our states-to-skip-on list. "
+                    "Skipping."
+                )
+
+        # only go green if every single task has reached an allowed state
+        count_allowed = self.get_count(dttm_filter, session, self.allowed_states)
         return count_allowed == len(dttm_filter)
 
     def _check_for_existence(self, session) -> None:
@@ -308,10 +365,10 @@ class ExternalTaskSensor(BaseSensorOperator):
                 .scalar()
             ) / len(self.external_task_ids)
         elif self.external_task_group_id:
-            external_task_group_task_ids = self.get_external_task_group_task_ids(session)
+            external_task_group_task_ids = self.get_external_task_group_task_ids(session, dttm_filter)
             count = (
                 self._count_query(TI, session, states, dttm_filter)
-                .filter(TI.task_id.in_(external_task_group_task_ids))
+                .filter(tuple_in_condition((TI.task_id, TI.map_index), external_task_group_task_ids))
                 .scalar()
             ) / len(external_task_group_task_ids)
         else:
@@ -321,21 +378,27 @@ class ExternalTaskSensor(BaseSensorOperator):
     def _count_query(self, model, session, states, dttm_filter) -> Query:
         query = session.query(func.count()).filter(
             model.dag_id == self.external_dag_id,
-            model.state.in_(states),  # pylint: disable=no-member
+            model.state.in_(states),
             model.execution_date.in_(dttm_filter),
         )
         return query
 
-    def get_external_task_group_task_ids(self, session):
+    def get_external_task_group_task_ids(self, session, dttm_filter):
         refreshed_dag_info = DagBag(read_dags_from_db=True).get_dag(self.external_dag_id, session)
         task_group = refreshed_dag_info.task_group_dict.get(self.external_task_group_id)
 
         if task_group:
-            return [task.task_id for task in task_group]
+            group_tasks = session.query(TaskInstance).filter(
+                TaskInstance.dag_id == self.external_dag_id,
+                TaskInstance.task_id.in_(task.task_id for task in task_group),
+                TaskInstance.execution_date.in_(dttm_filter),
+            )
+
+            return [(t.task_id, t.map_index) for t in group_tasks]
 
         # returning default task_id as group_id itself, this will avoid any failure in case of
         # 'check_existence=False' and will fail on timeout
-        return [self.external_task_group_id]
+        return [(self.external_task_group_id, -1)]
 
     def _handle_execution_date_fn(self, context) -> Any:
         """
