@@ -20,13 +20,17 @@ import logging
 import warnings
 from os import path
 
-from connexion import App, ProblemException
+from connexion import FlaskApi, ProblemException, Resolver
+from connexion.decorators.validation import RequestBodyValidator
+from connexion.exceptions import BadRequestProblem
 from flask import Flask, request
 
 from airflow.api_connexion.exceptions import common_error_handler
+from airflow.compat.functools import cached_property
 from airflow.configuration import conf
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.security import permissions
+from airflow.utils.yaml import safe_load
 
 log = logging.getLogger(__name__)
 
@@ -149,13 +153,6 @@ def init_plugins(app):
         app.register_blueprint(blue_print["blueprint"])
 
 
-def init_connection_form():
-    """Initializes connection form"""
-    from airflow.www.views import lazy_add_provider_discovered_options_to_connection_form
-
-    lazy_add_provider_discovered_options_to_connection_form()
-
-
 def init_error_handlers(app: Flask):
     """Add custom errors handlers"""
     from airflow.www import views
@@ -183,6 +180,51 @@ def set_cors_headers_on_response(response):
     return response
 
 
+class _LazyResolution:
+    """OpenAPI endpoint that lazily resolves the function on first use.
+
+    This is a stand-in replacement for ``connexion.Resolution`` that implements
+    its public attributes ``function`` and ``operation_id``, but the function
+    is only resolved when it is first accessed.
+    """
+
+    def __init__(self, resolve_func, operation_id):
+        self._resolve_func = resolve_func
+        self.operation_id = operation_id
+
+    @cached_property
+    def function(self):
+        return self._resolve_func(self.operation_id)
+
+
+class _LazyResolver(Resolver):
+    """OpenAPI endpoint resolver that loads lazily on first use.
+
+    This re-implements ``connexion.Resolver.resolve()`` to not eagerly resolve
+    the endpoint function (and thus avoid importing it in the process), but only
+    return a placeholder that will be actually resolved when the contained
+    function is accessed.
+    """
+
+    def resolve(self, operation):
+        operation_id = self.resolve_operation_id(operation)
+        return _LazyResolution(self.resolve_function_from_operation_id, operation_id)
+
+
+class _CustomErrorRequestBodyValidator(RequestBodyValidator):
+    """Custom request body validator that overrides error messages.
+
+    By default, Connextion emits a very generic *None is not of type 'object'*
+    error when receiving an empty request body (with the view specifying the
+    body as non-nullable). We overrides it to provide a more useful message.
+    """
+
+    def validate_schema(self, data, url):
+        if not self.is_null_value_valid and data is None:
+            raise BadRequestProblem(detail="Request body must not be empty")
+        return super().validate_schema(data, url)
+
+
 def init_api_connexion(app: Flask) -> None:
     """Initialize Stable API"""
     base_path = "/api/v1"
@@ -207,24 +249,48 @@ def init_api_connexion(app: Flask) -> None:
         else:
             return views.method_not_allowed(ex)
 
-    spec_dir = path.join(ROOT_APP_DIR, "api_connexion", "openapi")
-    swagger_ui_dir = path.join(ROOT_APP_DIR, "www", "static", "dist", "swagger-ui")
-    options = {
-        "swagger_ui": conf.getboolean("webserver", "enable_swagger_ui", fallback=True),
-        "swagger_path": swagger_ui_dir,
-    }
-    connexion_app = App(__name__, specification_dir=spec_dir, skip_error_handlers=True, options=options)
-    connexion_app.app = app
-    api_bp = connexion_app.add_api(
-        specification="v1.yaml", base_path=base_path, validate_responses=True, strict_validation=True
+    with open(path.join(ROOT_APP_DIR, "api_connexion", "openapi", "v1.yaml")) as f:
+        specification = safe_load(f)
+    api_bp = FlaskApi(
+        specification=specification,
+        resolver=_LazyResolver(),
+        base_path=base_path,
+        options={
+            "swagger_ui": conf.getboolean("webserver", "enable_swagger_ui", fallback=True),
+            "swagger_path": path.join(ROOT_APP_DIR, "www", "static", "dist", "swagger-ui"),
+        },
+        strict_validation=True,
+        validate_responses=True,
+        validator_map={"body": _CustomErrorRequestBodyValidator},
     ).blueprint
-    # Like "api_bp.after_request", but the BP is already registered, so we have
-    # to register it in the app directly.
-    app.after_request_funcs.setdefault(api_bp.name, []).append(set_cors_headers_on_response)
+    api_bp.after_request(set_cors_headers_on_response)
+
+    app.register_blueprint(api_bp)
     app.register_error_handler(ProblemException, common_error_handler)
     app.extensions["csrf"].exempt(api_bp)
 
 
+def init_api_internal(app: Flask, standalone_api: bool = False) -> None:
+    """Initialize Internal API"""
+    if not standalone_api and not conf.getboolean("webserver", "run_internal_api", fallback=False):
+        return
+
+    with open(path.join(ROOT_APP_DIR, "api_internal", "openapi", "internal_api_v1.yaml")) as f:
+        specification = safe_load(f)
+    api_bp = FlaskApi(
+        specification=specification,
+        base_path="/internal_api/v1",
+        options={"swagger_ui": conf.getboolean("webserver", "enable_swagger_ui", fallback=True)},
+        strict_validation=True,
+        validate_responses=True,
+    ).blueprint
+    api_bp.after_request(set_cors_headers_on_response)
+
+    app.register_blueprint(api_bp)
+    app.after_request_funcs.setdefault(api_bp.name, []).append(set_cors_headers_on_response)
+    app.extensions["csrf"].exempt(api_bp)
+
+
 def init_api_experimental(app):
     """Initialize Experimental API"""
     if not conf.getboolean("api", "enable_experimental_api", fallback=False):
